/*
 * Website: http://www.crimson-blade.org/
 * File: nations.c
 * Name: Standalone Racial Nations Module
 * Version: 2.13
 * Author: John 'Noplex' Bellone (jbellone@comcast.net)
 * Concent and Design:
           John 'Noplex' Bellone
           Chris 'Syriac' Crowe  (krowe69@hotmail.com)
 * Terms:
 * File may be modified or redistributed as long as the author is
 * contacted via email or other means. Description may be altered
 * to fit the needs. This header should be kept intact as it is
 * except for the option to alter the description.
 * Description:
 * Standalone version of the Nations module. Allows for the Nation
 * leaders to set positions, including the Nations First and Second.
 * Nations can be created or destroyed by Immortals; whom also set
 * the Nation Leader of each nation.
 */

#include <sys/types.h>
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "mud.h"

/* Macros */
#ifdef KEY
 #undef KEY
#endif
#define KEY( literal, field, value )                                    \
                                if ( !strcmp( word, literal ) )         \
                                {                                       \
                                    field  = value;                     \
                                    fMatch = TRUE;                      \
                                    break;                              \
                                }
#ifdef FCLOSE
 #undef FCLOSE
#endif
#define FCLOSE(fp) fclose(fp); fp=NULL;


#ifdef NULLSTR
 #undef NULLSTR
#endif
#define NULLSTR(str) (!str || str[0] == '\0')

/* Globals */
int         top_nation;

/* Local */
static char *nation_rank_strings[] =
 { "&g%s Leader&G" };

/* save nations to the disk -Nopey */
void save_nations(void)
{
  FILE *fp = NULL;
  int i;

  if((fp = fopen(NATION_LIST, "w")) == NULL)
  {
    bug("save_nations(): cannot open %s for writing", NATION_LIST);
    return;
  }

  for(i=0; i < MAX_NATIONS; i++)
   if(nation_list[i] && !NULLSTR(nation_list[i]->name))
   {
    fprintf(fp, "%s.nation\n", nation_list[i]->name);
    save_nation_file(nation_list[i]);
    continue;
   }
  fprintf(fp, "$\n");
  FCLOSE(fp);
  return;
}

/* save an invididual nation file -Nopey */
void save_nation_file(NATION_DATA *nation)
{
  FILE *fp = NULL;
  char filename[256];
  int i;

  sprintf(filename, "%s%s.nation", NATION_DIR, nation->name);
  if((fp = fopen(filename, "w")) == NULL)
  {
    bug("save_nation_file(): cannot open %s for writing", filename);
    return;
  }
  
  if(NULLSTR(nation->name))
  {
    bug("save_nation_file: nation has no name! (%s)", filename);
    return;
  }
  else
   fprintf(fp, "Name          %s~\n", nation->name);
  if(!NULLSTR(nation->leader))
   fprintf(fp, "Leader        %s~\n", nation->leader);
  if(!NULLSTR(nation->first))
   fprintf(fp, "First         %s~\n", nation->first);
  if(!NULLSTR(nation->second))
   fprintf(fp, "Second        %s~\n", nation->second);
  if(!NULLSTR(nation->description))
   fprintf(fp, "Description   \n%s~\n", nation->description);
  fprintf(fp, "Race           %d\n", nation->race);
  for(i=0; i < MAX_NATION_RANKS; i++)
   if(nation->ranklist[i] && (!NULLSTR(nation->ranklist[i]->name) && !NULLSTR(nation->ranklist[i]->holder)))
    fprintf(fp, "RANK %s~ %s~\n", nation->ranklist[i]->name, nation->ranklist[i]->holder);
  fprintf(fp, "End\n");
  FCLOSE(fp);
  return;
}

/* load the nations from the files, etc -Nopey */
void load_nations(void)
{
  FILE *fp = NULL;

  if((fp = fopen(NATION_LIST, "r")) == NULL)
  {
    bug("load_nations(): cannot open %s for reading", NATION_LIST);
    return;
  }
  
  for( ; ; )
  {
    char *file = feof(fp) ? "$" : fread_word(fp);

    if(file[0] == '$')
     break;
    else
    {
      NATION_DATA *nation = NULL;

      if((nation = fread_nation(file)) == NULL)
      {
      	bug("load_nations(): fread_nation returned NULL %s", file);
      	continue;
      }
      nation_list[top_nation] = nation;
      top_nation++;
      continue;
    }
  }
  FCLOSE(fp);
  return;
}

/* read in a nation; open up the file from disk -Nopey */
NATION_DATA *fread_nation(char *file)
{
  FILE *fp = NULL;
  char filename[256];
  NATION_DATA *nation = NULL;
  bool fMatch = FALSE;

  sprintf(filename, "%s%s", NATION_DIR, file);
  if((fp = fopen(filename, "r")) == NULL)
  {
    bug("fread_nation(): cannot open %s for reading", filename);
    return NULL;
  }
  
  CREATE(nation, NATION_DATA, 1);
  nation->name                = STRALLOC("");
  nation->leader              = STRALLOC("None");
  nation->first               = STRALLOC("None");
  nation->second              = STRALLOC("None");
  nation->description         = STRALLOC("None");
  nation->race                = -1;
  nation->top_rank            = 0;

  for( ; ; )
  {
    char *word = feof(fp) ? "End" : fread_word(fp);
    
    switch(UPPER(word[0]))
    {
    	case '*':
    	 fread_to_eol(fp);
    	break;
    	
    	case 'D':
    	 KEY("Description", nation->description, fread_string(fp));
    	break;
    	
    	case 'E':
    	 if(!str_cmp(word, "End"))
    	 {
    	   FCLOSE(fp);
           if(NULLSTR(nation->name))
            return NULL;
           return nation;
         }
        break;

    	case 'F':
    	 KEY("First", nation->first, fread_string(fp));
    	break;

        case 'L':
         KEY("Leader", nation->leader, fread_string(fp));
        break;

    	case 'N':
    	 KEY("Name", nation->name, fread_string(fp));
    	break;

        case 'S':
         KEY("Second", nation->second, fread_string(fp));
        break;
        
        case 'R':
         KEY("Race", nation->race, fread_number(fp));
         if(!str_cmp(word, "RANK"))
         {
           NRANKLIST *rank = NULL;
           
           CREATE(rank, NRANKLIST, 1);
           rank->name       = fread_string(fp);
           rank->holder     = fread_string(fp);
           
           if(!NULLSTR(rank->name) && !NULLSTR(rank->holder))
           {
             nation->ranklist[nation->top_rank] = rank;
             nation->top_rank++;
           }
           break;
         }
        break;
    }
    if(!fMatch)
     bug("fread_nation(): no match for %s", word);
  }
}

/* get a nation based on the nation or race name -Nopey */
NATION_DATA *get_nation(char *check)
{
  int i;
  
  for(i=0; i < MAX_NATIONS; i++)
   if(nation_list[i] && (!str_cmp(nation_list[i]->name, check)
    || (nation_list[i]->race != -1 && !str_cmp(race_table[nation_list[i]->race]->race_name, check))))
    return nation_list[i];

  return NULL;
}

/* nation leader check -Nopey */
NATION_DATA *is_nation_leader(CHAR_DATA *ch)
{
  int i;
  
  for(i=0; i < MAX_NATIONS; i++)
   if(nation_list[i] && !str_cmp(nation_list[i]->leader, ch->name))
    return nation_list[i];
  return NULL;
}

/* nations command; view each individual nation; view a list of them -Nopey */
void do_nations(CHAR_DATA *ch, char *argument)
{
  if(!NULLSTR(argument))
  {
    NATION_DATA *nation = get_nation(argument);
    
    if(!nation)
    {
      send_to_char("No such nation.\n\r", ch);
      send_to_char_color("&GUse 'nations' to for a complete list of nations.&g\n\r", ch);
      return;
    }
    view_nation(ch, FALSE, nation);
    send_to_char_color("&GUse 'nations' to for a complete list of nations.&g\n\r", ch);
    return;
  }
  else
  {
    view_nation(ch, TRUE, NULL);
    send_to_char_color("&GUse nations '<race>' or nations '<nation>' for specifics.&g\n\r", ch);
    return;
  }
}

/* function that prints nation information to characters -Nopey */
void view_nation(CHAR_DATA *ch, bool all, NATION_DATA *nation)
{
  if(!ch)
  {
    bug("%s","view_nation(): no character");
    return;
  }
  
  if(!all && nation)
  {
    int i;

    pager_printf_color(ch, "&GNation:&W\t%s\n\r", nation->name);
    if(!NULLSTR(nation->leader))
     pager_printf_color(ch, "&GLeader:&W\t%s\n\r", nation->leader);
    if(!NULLSTR(nation->first))
     pager_printf_color(ch, "&GFirst:&W\t%s\n\r", nation->first);
    if(!NULLSTR(nation->second))
     pager_printf_color(ch, "&GSecond:&W\t%s\n\r", nation->second);
    send_to_pager_color("&GNation Positions\n\r", ch);
    for(i=0; i < MAX_NATION_RANKS; i++)
     if(nation->ranklist[i] && !NULLSTR(nation->ranklist[i]->name) && !NULLSTR(nation->ranklist[i]->holder))
     {
       pager_printf_color(ch, "&G%s:&W\t", capitalize(nation->ranklist[i]->name));
       pager_printf_color(ch, "%s\n\r", capitalize(nation->ranklist[i]->holder));
     }
    if(!NULLSTR(nation->description))
     pager_printf_color(ch, "&G\n\rDescription:&W\n\r%s&g\n\r\n\r", nation->description);
  }
  else if(all)
  {
    int i;
    
    send_to_pager_color("&G\n\rNation                Leader          Race\n\r", ch);
    send_to_pager_color("&g____________________________________________\n\r\n\r", ch);
    for(i=0; i < MAX_NATIONS; i++)
     if(nation_list[i] && !NULLSTR(nation_list[i]->name))
      pager_printf_color(ch, "&G&W%-22s %-14s %s&g\n\r", capitalize(nation_list[i]->name), nation_list[i]->leader ? nation_list[i]->leader : "None", nation_list[i]->race != -1 ? race_table[nation_list[i]->race]->race_name : "None");
    send_to_pager_color("&g____________________________________________\n\r\n\r", ch);
  }
  else
  {
    bug("view_nation(): NULL nation from %s", ch->name);
  }
  return;
}

/* set nation command -Nopey */
void do_setnation(CHAR_DATA *ch, char *argument)
{
  char arg[MAX_INPUT_LENGTH];

  argument = one_argument(argument, arg);

  if(IS_NPC(ch) || !IS_IMMORTAL(ch))
  {
    send_to_char("Huh?\n\r", ch);
    return;
  }
  
  set_char_color(AT_YELLOW, ch);
  if(NULLSTR(arg))
  {
    send_to_char("Syntax: setnation save\n\r", ch);
    send_to_char("Syntax: setnation create <nation>\n\r", ch);
    send_to_char("Syntax: setnation remove <nation>\n\r", ch);
    send_to_char("Syntax: setnation remrank <nation> <position-name/holder>\n\r", ch);
    send_to_char("Syntax: setnation <nation> <field> [value]\n\r", ch);
    send_to_char("Field being one of the following:\n\r"
                 " name leader second first race description\n\r", ch);
    return;
  }
  
  if(!str_cmp(arg, "create"))
  {
    NATION_DATA *nation = NULL;

    if(top_nation >= MAX_NATIONS)
    {
      send_to_char("Nations have already reached the maximum allocated.\n\r", ch);
      return;
    }

    if(NULLSTR(argument))
    {
      send_to_char("Syntax: setnation create <nation>\n\r", ch);
      return;
    }

    if(get_nation(argument))
    {
      send_to_char("That nation has already been created.\n\r", ch);
      return;
    }

    CREATE(nation, NATION_DATA, 1);
    nation->name                = STRALLOC(argument);
    nation->leader              = STRALLOC("None");
    nation->first               = STRALLOC("None");
    nation->second              = STRALLOC("None");
    nation->description         = STRALLOC("None");
    nation->race                = -1;
    nation->top_rank            = 0;

    nation_list[top_nation] = nation;
    top_nation++;
    send_to_char("Done. Nation created.\n\r", ch);
    save_nations();
    return;
  }
  else if(!str_cmp(arg, "remove"))
  {
    NATION_DATA *nation = NULL;
    int i;
    
    if(NULLSTR(argument))
    {
      send_to_char("Syntax: setnation remove <nation-name>\n\r", ch);
      return;
    }
    
    if((nation = get_nation(argument)) == NULL)
    {
      send_to_char("No such nation has been created.\n\r", ch);
      return;
    }
    
    if(!NULLSTR(nation->name))
     STRFREE(nation->name);
    if(!NULLSTR(nation->leader))
     STRFREE(nation->leader);
    if(!NULLSTR(nation->first))
     STRFREE(nation->first);
    if(!NULLSTR(nation->second))
     STRFREE(nation->second);
    if(!NULLSTR(nation->description))
     STRFREE(nation->description);
    for(i=0; i < MAX_NATION_RANKS; i++)
     if(nation->ranklist[i])
     {
     	if(!NULLSTR(nation->ranklist[i]->name))
     	 STRFREE(nation->ranklist[i]->name);
     	if(!NULLSTR(nation->ranklist[i]->holder))
     	 STRFREE(nation->ranklist[i]->holder);
     	DISPOSE(nation->ranklist[i]);
     	nation->ranklist[i] = NULL;
     }
    for(i=0; i < MAX_NATIONS; i++)
     if(nation == nation_list[i])
     {
      DISPOSE(nation);
      nation_list[i] = NULL;
      break;
     }
    send_to_char("Done. Nation removed.\n\r", ch);
    save_nations();
    return;
  }
  else if(!str_cmp(arg, "save"))
  {
    send_to_char("Done. Nations saved.\n\r", ch);
    save_nations( );
    return;
  }
  else if(!str_cmp(arg, "remrank"))
  {
    char arg2[MAX_INPUT_LENGTH];
    NATION_DATA *nation = NULL;
    int i;

    argument = one_argument(argument, arg2);

    if(NULLSTR(arg2) || NULLSTR(argument))
    {
      send_to_char("Syntax: setnation remrank <nation> <position/holder>\n\r", ch);
      send_to_char("To gain the 'position' use 'nations <nation>'\n\r", ch);
      return;
    }
    
    if((nation = get_nation(arg2)) == NULL)
    {
      send_to_char("No such nation exists.\n\r", ch);
      return;
    }
    
    for(i=0; i < MAX_NATION_RANKS; i++)
     if(nation->ranklist[i] && (!str_cmp(nation->ranklist[i]->name, argument)
      || !str_cmp(nation->ranklist[i]->holder, argument)))
     {
	if(!NULLSTR(nation->ranklist[i]->name))
	 STRFREE(nation->ranklist[i]->name);
	if(!NULLSTR(nation->ranklist[i]->holder))
	 STRFREE(nation->ranklist[i]->holder);
        DISPOSE(nation->ranklist[i]);
        nation->ranklist[i] = NULL;
	send_to_char("Done. Position removed.\n\r", ch);
	save_nation_file(nation);
	return;
     }
    send_to_char("No such nation position.\n\r", ch);
    return;
  }
  else
  {
    char arg2[MAX_INPUT_LENGTH];
    NATION_DATA *nation = NULL;
    
    argument = one_argument(argument, arg2);
    if(NULLSTR(arg2) || NULLSTR(argument))
    {
      send_to_char("Syntax: setnation <nation-name> <field> [value]\n\r", ch);
      send_to_char("Field being one of the following:\n\r"
                   " name leader second first race description\n\r", ch);
      return;
    }
    
    if((nation = get_nation(arg)) == NULL)
    {
      send_to_char("No such nation exists.\n\r", ch);
      return;
    }
    
    if(!str_cmp(arg2, "name"))
    {
      nation->name = STRALLOC(argument);
      send_to_char("Done.\n\r", ch);
      save_nation_file(nation);
      return;
    }
    else if(!str_cmp(arg2, "leader"))
    {
      CHAR_DATA *vic = get_char_world(ch, argument);

      if(!str_cmp(argument, "clear") || !str_cmp(argument, "none"))
      {
      	CHAR_DATA *vict = get_char_world(ch, nation->leader);

      	if(vict && !IS_NPC(vict))
      	{
      	 vict->pcdata->rank = str_dup("");
      	 vict->pcdata->nation = NULL;
      	 vict->pcdata->nation_name = STRALLOC("");
      	 save_char_obj(vict);
        }

      	nation->leader = STRALLOC("None");
      	send_to_char("Done. Nation Leader cleared.\n\r", ch);
      	save_nation_file(nation);
      	return;
      }	

      if(!vic || IS_NPC(vic))
      {
      	send_to_char("No such character online.\n\r", ch);
      	return;
      }

      if(vic->race != nation->race)
      {
        send_to_char("That character is not of the right race for this position.\n\r", ch);
        return;
      }
      else
      {
        char buf[MAX_INPUT_LENGTH];
      
        sprintf(buf, nation_rank_strings[0], capitalize(nation->name));
      	nation->leader = STRALLOC(capitalize(vic->name));
        vic->pcdata->rank        = str_dup(buf);
        vic->pcdata->nation_name = STRALLOC(nation->name);
        vic->pcdata->nation      = nation;
        send_to_char("Done.\n\r", ch);
        send_to_char("You are now Nation Leader!\n\r", vic);
        save_nation_file(nation);
        save_char_obj(vic);
        return;
      }
    }
    else if(!str_cmp(arg2, "first"))
    {
      CHAR_DATA *vic = get_char_world(ch, argument);
     
      if(!str_cmp(argument, "clear") || !str_cmp(argument, "none"))
      {
      	/*
        CHAR_DATA *vict = get_char_world(ch, nation->first);

      	if(vict && !IS_NPC(vict))
      	{
      	 vict->pcdata->rank = str_dup("");
     	 vict->pcdata->nation = NULL;
      	 vict->pcdata->nation_name = STRALLOC("");
      	 save_char_obj(vict);
        }
        */
      	nation->first = STRALLOC("None");
      	send_to_char("Done. Nation's First cleared.\n\r", ch);
      	save_nation_file(nation);
      	return;
      }

      if(!vic || IS_NPC(vic))
      {
      	send_to_char("No such character online.\n\r", ch);
      	return;
      }

      if(vic->race != nation->race)
      {
        send_to_char("That character is not of the right race for this position.\n\r", ch);
        return;
      }
      else
      {
        /*
        char buf[MAX_INPUT_LENGTH];

        sprintf(buf, nation_rank_strings[1], capitalize(nation->name));
      	nation->first = STRALLOC(capitalize(vic->name));
        vic->pcdata->rank        = str_dup(buf);
        */

        vic->pcdata->nation_name = STRALLOC(nation->name);
        vic->pcdata->nation      = nation;
        send_to_char("Done.\n\r", ch);
        send_to_char("You are now Nation's First!\n\r", vic);
        save_nation_file(nation);
        save_char_obj(vic);
        return;
      }
      send_to_char("No such character online.\n\r", ch);
      return;
    }
    else if(!str_cmp(arg2, "second"))
    {
      CHAR_DATA *vic = get_char_world(ch, argument);

      if(!str_cmp(argument, "clear") || !str_cmp(argument, "none"))
      {
      	/*
      	CHAR_DATA *vict = get_char_world(ch, nation->second);

      	if(vict && !IS_NPC(vict))
      	{
      	 vict->pcdata->rank = str_dup("");
      	 vict->pcdata->nation = NULL;
      	 vict->pcdata->nation_name = STRALLOC("");
      	 save_char_obj(vict);
      	}
      	*/
      	nation->second = STRALLOC("None");
      	send_to_char("Done. Nation's Second cleared.\n\r", ch);
      	save_nation_file(nation);
      	return;
      }

      if(!vic || IS_NPC(vic))
      {
      	send_to_char("No such character online.\n\r", ch);
      	return;
      }

      if(vic->race != nation->race)
      {
        send_to_char("That character is not of the right race for this position.\n\r", ch);
        return;
      }
      else
      {
      	/*
        char buf[MAX_INPUT_LENGTH];

        sprintf(buf, nation_rank_strings[2], capitalize(nation->name));
      	nation->second = STRALLOC(capitalize(vic->name));
        vic->pcdata->rank        = str_dup(buf);
        */

        vic->pcdata->nation_name = STRALLOC(nation->name);
        vic->pcdata->nation      = nation;
        send_to_char("Done.\n\r", ch);
        send_to_char("You are now Nation's Second!\n\r", vic);
        save_nation_file(nation);
        save_char_obj(vic);
        return;
      }
      send_to_char("No such character online.\n\r", ch);
      return;
    }
    else if(!str_cmp(arg2, "description"))
    {
      smash_tilde(argument);
      nation->description = STRALLOC(argument);
      send_to_char("Done.\n\r", ch);
      save_nation_file(nation);
      return;
    }
    else if(!str_cmp(arg2, "race"))
    {
      int i;

      for(i=0; i < MAX_PC_RACE; i++)
       if(!str_cmp(argument, race_table[i]->race_name))
       {
       	 nation->race = i;
       	 send_to_char("Done.\n\r", ch);
       	 save_nation_file(nation);
       	 return;
       }
      send_to_char("No such race.\n\r", ch);
      return;
    }
  }
  do_setnation(ch, "");
  return;
}

/* nation leader edit function -Nopey */
void do_editnation(CHAR_DATA *ch, char *argument)
{
  NATION_DATA *nation = is_nation_leader(ch);
  char arg[MAX_INPUT_LENGTH];

  argument = one_argument(argument, arg);

  if(IS_NPC(ch) || !nation)
  {
    send_to_char("Huh?\n\r", ch);
    return;
  }

  if(NULLSTR(arg) || NULLSTR(argument))
  {
    send_to_char("Syntax: editnation first <name>\n\r", ch);
    send_to_char("Syntax: editnation second <name>\n\r", ch);
    send_to_char("Syntax: editnation add '<position>' <name>\n\r", ch);
    send_to_char("Syntax: editnation rem '<position>'\n\r", ch);
    return;
  }

  if(!str_cmp(arg, "first"))
  {
    CHAR_DATA *vic = get_char_world(ch, argument);

    if(!str_cmp(argument, "clear") || !str_cmp(argument, "none"))
    {
    	CHAR_DATA *vict = get_char_world(ch, nation->first);

    	if(vict && !IS_NPC(vict))
    	{
    	 /*
    	 vict->pcdata->rank = str_dup("");
    	 */
    	 vict->pcdata->nation = NULL;
    	 vict->pcdata->nation_name = STRALLOC("");
    	 save_char_obj(vict);
    	}
    	nation->first = STRALLOC("None");
    	send_to_char("Done. Nation's First cleared.\n\r", ch);
    	save_nation_file(nation);
    	return;
    }

    if(!vic || IS_NPC(vic))
    {
    	send_to_char("No such character online.\n\r", ch);
    	return;
    }

    if(vic->race != nation->race)
    {
      send_to_char("That character is not of the right race for this position.\n\r", ch);
      return;
    }
    else
    {
      /*
      char buf[MAX_INPUT_LENGTH];

      sprintf(buf, nation_rank_strings[1], capitalize(nation->name));
      nation->first = STRALLOC(capitalize(vic->name));
      vic->pcdata->rank        = str_dup(buf);
      */

      vic->pcdata->nation_name = STRALLOC(nation->name);
      vic->pcdata->nation      = nation;
      send_to_char("Done.\n\r", ch);
      send_to_char("You are now Nation's First!\n\r", vic);
      save_nation_file(nation);
      save_char_obj(vic);
      return;
    }
    send_to_char("No such character online.\n\r", ch);
    return;
  }
  else if(!str_cmp(arg, "second"))
  {
    CHAR_DATA *vic = get_char_world(ch, argument);

    if(!str_cmp(argument, "clear") || !str_cmp(argument, "none"))
    {
    	CHAR_DATA *vict = get_char_world(ch, nation->second);

    	if(vict && !IS_NPC(vict))
    	{
    	 /*
    	 vict->pcdata->rank = str_dup("");
    	 */
    	 vict->pcdata->nation = NULL;
    	 vict->pcdata->nation_name = STRALLOC("");
    	 save_char_obj(vict);
    	}
    	nation->second = STRALLOC("None");
    	send_to_char("Done. Nation's Second cleared.\n\r", ch);
    	save_nation_file(nation);
    	return;
    }

    if(!vic || IS_NPC(vic))
    {
    	send_to_char("No such character online.\n\r", ch);
    	return;
    }

    if(vic->race != nation->race)
    {
      send_to_char("That character is not of the right race for this position.\n\r", ch);
      return;
    }
    else
    {
      /*
      char buf[MAX_INPUT_LENGTH];

      sprintf(buf, nation_rank_strings[2], capitalize(nation->name));
      nation->second = STRALLOC(capitalize(vic->name));
      vic->pcdata->rank        = str_dup(buf);
      */

      vic->pcdata->nation_name = STRALLOC(nation->name);
      vic->pcdata->nation      = nation;
      send_to_char("Done.\n\r", ch);
      send_to_char("You are now the Nation's Second!\n\r", vic);
      save_nation_file(nation);
      save_char_obj(vic);
      return;
    }
    send_to_char("No such character online.\n\r", ch);
    return;
  }
  else if(!str_cmp(arg, "add"))
  {
    int i;
    char arg2[MAX_INPUT_LENGTH];
    NRANKLIST *rank = NULL;
    
    argument = one_argument(argument, arg2);
   
    if(nation->top_rank >= MAX_NATION_RANKS)
    {
      send_to_char("Nation positions have already reached the maximum allocated.\n\r", ch);
      send_to_char("Remove a position to add another.\n\r", ch);
      return;
    }

    if(NULLSTR(argument) || NULLSTR(arg2))
    {
      send_to_char("Syntax: editnation add <position> <name>\n\r", ch);
      return;
    }
    
    for(i=0; i < MAX_NATION_RANKS; i++)
     if(nation->ranklist[i] && !str_cmp(nation->ranklist[i]->name, arg2))
     {
       send_to_char("That rank already exists.\n\r", ch);
       return;
     }
     else if(nation->ranklist[i] && !str_cmp(nation->ranklist[i]->holder, argument))
     {
       send_to_char("That player already has a rank.\n\r", ch);
       return;
     }
    CREATE(rank, NRANKLIST, 1);
    rank->name              = STRALLOC(arg2);
    rank->holder            = STRALLOC(argument);

    nation->ranklist[nation->top_rank] = rank;
    nation->top_rank++;
    send_to_char("Done. Position added.\n\r", ch);
    save_nation_file(nation);
    return;
  }
  else if(!str_cmp(arg, "rem"))
  {
    int i;
    
    for(i=0; i < MAX_NATION_RANKS; i++)
     if(nation->ranklist[i] && !str_cmp(argument, nation->ranklist[i]->name))
     {
       if(!NULLSTR(nation->ranklist[i]->name))
        STRFREE(nation->ranklist[i]->name);
       if(!NULLSTR(nation->ranklist[i]->holder))
        STRFREE(nation->ranklist[i]->holder);
       DISPOSE(nation->ranklist[i]);
       nation->ranklist[i] = NULL;
       send_to_char("Done. Position removed.\n\r", ch);
       save_nation_file(nation);
       return;
     }
    send_to_char("No such position.\n\r", ch);
    return;
  }
  do_editnation(ch, "");
  return;
}

/* check the rank status of a character to see if they
 * were a nation leader and should have their rank changed -Nopey */
void check_nation_status(CHAR_DATA *ch)
{
  /* null rank */
  if(NULLSTR(ch->pcdata->rank))
   return;

  if(!NULLSTR(ch->pcdata->nation_name))
  {
    NATION_DATA *nation = get_nation(ch->pcdata->nation_name);
    
    if(!nation)
    {
     STRFREE(ch->pcdata->nation_name);
     ch->pcdata->nation = NULL;
    }
    else
     ch->pcdata->nation = nation;
  }

  /* filter out ranks first */
  if(str_cmp(ch->pcdata->rank, nation_rank_strings[0]))
   return;
  else
  {
    NATION_DATA *nation = get_nation(race_table[ch->race]->race_name);

    if(!nation)
    {
      bug("check_nation_status(): character has nation title but race has no nation %s", ch->name);
      ch->pcdata->rank = str_dup("");
      return;
    }

    if(str_cmp(nation->leader, ch->name) && str_cmp(nation->first, ch->name)
     && str_cmp(nation->second, ch->name))
     {
     	ch->pcdata->rank = str_dup("");
     	ch->pcdata->nation = NULL;
     	ch->pcdata->nation_name = STRALLOC("");
     	save_char_obj(ch);
     	return;
     }
  }
}

/* Allow player to declare what nation they belong to -Nopey */
void do_fealty(CHAR_DATA *ch, char *argument)
{
  char arg[MAX_INPUT_LENGTH];

  argument = one_argument(argument, arg);

  if(IS_NPC(ch))
  {
    send_to_char("Huh?\n\r", ch);
    return;
  }
  
  if(NULLSTR(arg))
  {
    if(ch->pcdata->nation)
    {
      pager_printf_color(ch, "&GYour alligance belongs to the %s.\n\r", capitalize(ch->pcdata->nation->name));
      send_to_char_color("&GYou can &grenounce&G your alligance to a nation by typing '&gfealty renounce&G'.\n\r", ch);
    }
    else
    {
      send_to_char_color("&GNo nation has your alligance.\n\r", ch);
      send_to_char_color("&GYou can &gswear&G your alligance to a nation by typing '&gfealty swear <nation>&G'.\n\r", ch);
    }
    return;
  }
  
  if(!str_cmp(arg, "renounce"))
  {
     if(!ch->pcdata->nation)
     {
     	send_to_char_color("&GYou have no alligance with a nation to begin with.\n\r", ch);
     	return;
     }
     
     if(is_nation_leader(ch))
     {
     	send_to_char_color("&GYou cannot give up your alligance to your nation; You are their leader!\n\r", ch);
     	return;
     }
     else if(!str_cmp(ch->pcdata->nation->first, ch->name) || !str_cmp(ch->pcdata->nation->second, ch->name))
     {
     	send_to_char_color("&GYou cannot give up your alligance to your Nation; You are an official!\n\r", ch);
     	return;
     }
     pager_printf_color(ch, "&GYou have renounced your alligance to the %s.", capitalize(ch->pcdata->nation->name));
     ch->pcdata->nation = NULL;
     ch->pcdata->nation_name[0] = '\0';
     return;
  }
  else if(!str_cmp(arg, "swear"))
  {
    NATION_DATA *nation = NULL;
    char buf[MAX_INPUT_LENGTH];

    if(NULLSTR(argument))
    {
      do_fealty(ch, "");
      return;
    }
    
    if(ch->pcdata->nation)
    {
      send_to_char_color("&GYou cannot give your alligance to more than one nation.\n\r", ch);
      return;
    }

    if((nation = get_nation(argument)) == NULL)
    {
      send_to_char_color("&GThere is no nation that goes by that name.\n\r", ch);
      return;
    }
    ch->pcdata->nation      = nation;
    ch->pcdata->nation_name = STRALLOC(nation->name);
    save_char_obj(ch);
    sprintf(buf, "&GYou swear your alligance to %s and the %s people.\n\r", nation->leader, nation->name);
    pager_printf_color(ch, buf);
    return;
  }
  do_fealty(ch, "");
  return;
}

